package frc.robot.subsystems;

import com.ctre.phoenix.motorcontrol.can.WPI_VictorSPX;

import edu.wpi.first.math.controller.ElevatorFeedforward;
import edu.wpi.first.math.controller.PIDController;
import edu.wpi.first.wpilibj.Encoder;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.SubsystemBase;
import frc.robot.constants.ElevatorConstants;

public class Elevator extends SubsystemBase {
    private WPI_VictorSPX motor1;
    private WPI_VictorSPX motor2;

    private Encoder encoder;

    private PIDController pidController;
    private ElevatorFeedforward feedforward;

    public Elevator() {
        motor1 = new WPI_VictorSPX(0);
        motor2 = new WPI_VictorSPX(1);

        motor2.follow(motor1);

        encoder = new Encoder(0, 0);

        pidController = new PIDController(
            0, 
            0, 
            0
        );
        
        feedforward = new ElevatorFeedforward(
            0, 
            0, 
            0
        );
    }
    
    public Command manualControl(double input) {
        return run(() -> {
            motor1.setVoltage(
                pidController.calculate(
                    encoder.getRate(), 
                    ElevatorConstants.kMaxVelo * input
                ) + feedforward.calculate(
                    ElevatorConstants.kMaxVelo * input, 
                    0.0
                )
            );
        });
    }

    public Command goToSetpoint(int input) {
        return run(() -> {
            motor1.setVoltage(
                pidController.calculate(
                    encoder.getDistance(), 
                    input
                ) + feedforward.calculate(
                    input, 
                    0.0
                )
            );
        });
    }

    public Command holdElevator() {
        
        return run(() -> {
            motor1.setVoltage(
                feedforward.calculate(
                    encoder.getDistance(), 
                    0.0
                )
            );
        });
    }
}
